# Shell to Python Continuous Deployment

Get a common set of Python based CI / CD commands\
from various contexts of CA, DNS, CI and OS / OCI.

---

## Table Of Contents {#toc}

* 1 [Why](#why)
* 2 [How](#how)
  * 2.1 [Group environment variables](#how-variables)
  * 2.2 [Workflow files](#how-workflows)
  * 2.3 [Integration & Deployment](#how-actions)
* 3 [What](#what)
  * 3.1 [Features](#what-features)
  * 3.2 [Environment variables](#what-variables)
  * 3.3 [Operating Systems containers](#what-systems)
* 4 [Who](#who)
* 5 [Where](#where)
  * 5.1 [Chat](#where-chat)
  * 5.2 [Repo](#where-repo)
  * 5.3 [Site](#where-site)
* 6 [When](#when)
  * 6.1 [Release tasks](#when-release)
  * 6.2 [Further tasks](#when-further)

---

## 1 [Why](#toc) {#why}

Project workflows can occur in:
* an online or offline context, dealing with custom CA & DNS servers
* various Continuous Integration platforms, with different workflow syntaxes
* specific Operating Systems, having different commands and package names
* specific containers for JavaScript based actions (ForgeJo, Gitea, GitHub)

What if there was a unified way to do so?

---

## 2 [How](#toc) {how}

Picture it…

### 2.1 [Group environment variables](#how) {#how-variables}

* numbered
  * Certificate Authority armored certificates
  * Domain Name System servers IP addresses
* ref names for deployment contexts of feature, staging & release
* alternate locations of
  * Operating Systems & Software packages
  * Git repositories of
    * this project
    * its parent framework
    * projects not available in some distributions
      * shunit2
* SSH key & hosts for deployments
* a standalone POSIX shell script payload

### 2.2 [Workflow files](#how) {#how-workflows}

* calling the POSIX shell script payload as their very first step
* using a unified YAML syntax made of actions implemented in Python

### 2.3 [Integration & Deployment](#how) {#how-actions}

#### Shell

* [X] setting up the chosen Operating System container
  * [X] Domain Name System resolving
  * [X] package manager configuration & repositories
  * [X] Certificate Authorities trusting
  * [X] Operating System updating
* [X] installing Git to clone
  * [X] this project
  * [X] its parent project
* [X] installing Python
  * [X] system environment
  * [X] virtual environment
  * [ ] managed version
* [X] generating a Python module to switch context

#### Python

* [X] setting up an OpenSSH client
  * [X] known hosts
  * [ ] private key
* [ ] and finally parsing commands from
  * [X] system
  * [ ] workflow
* [ ] to run available actions
  * [ ] check project
  * [ ] build project
  * [X] browse workspace
  * [ ] synchronize

---

## 3 [What](#toc) {#what}

### 3.1 [Features](#what) {#what-features}

Handle project workflows in a unified way:

* whether the network infrastructure is
  * [X] online
  * [X] offline with custom
    * [X] Certificate Authorities
    * [X] Domain Name System servers

* whatever the Continuous Integration platform
  * [X] ForgeJo
  * [X] Gitea
  * [X] GitHub
  * [X] GitLab
  * [ ] SourceHut

* whatever the Operating System container
| System   | Latest                 | Previous               |
|:---------|:-----------------------|:-----------------------|
| Alma     | ☑ 9                    | ☑ 8                    |
| Alpine   | ☑ 3.20                 | ☑ 3.19                 |
| Arch     | ☑ 20240818 (.0.255804) | ☑ 20240101 (.0.204074) |
| Debian   | ☑ Bookworm (12)        | ☐ Bullseye (11)        |
| Fedora   | ☑ 40                   | ☑ 39                   |
| OpenSUSE | ☐ 15.6                 | ☐ 15.5                 |
| Rocky    | ☑ 9                    | ☑ 8                    |
| Ubuntu   | ☑ Noble (24.04)        | ☐ Jammy (22.04)        |

### 3.2 [Environment variables](#what) {#what-variables}

| Variable          | Description              | Default                         |
|:------------------|:-------------------------|:--------------------------------|
| SPCD_CA_n         | Numbered CA certificates |                                 |
| SPCD_DNS_n        | Numbered name servers    | 9.9.9.9                         |
| SPCD_GIT_RWX      | RWX Git repository       | rwx                             |
| SPCD_GIT_SHUNIT   | ShUnit Git repository    | shunit2                         |
| SPCD_GIT_SPCD     | SPCD Git repository      | spcd                            |
| SPCD_REF_ARCHIVE  | Archive deployment ref   | old                             |
| SPCD_REF_FEATURE  | Feature deployment ref   | f                               |
| SPCD_REF_RELEASE  | Release deployment ref   | main                            |
| SPCD_REF_STAGING  | Staging deployment ref   | dev                             |
| SPCD_SSH_HOSTS    | domain.tld ssh-type pub  |                                 |
| SPCD_SSH_KEY      | SSH private key          |                                 |
| SPCD_TXT_LOCALE   | Locale to use for text   | en_US                           |
| SPCD_URL_ALMA     | Alma repository URL      | https://repo.almalinux.org      |
| SPCD_URL_ALPINE   | Alpine repository URL    | https://dl-cdn.alpinelinux.org  |
| SPCD_URL_ARCH     | Arch repository URL      | https://geo.mirror.pkgbuild.com |
| SPCD_URL_DEBIAN   | Debian repository URL    | https://deb.debian.org          |
| SPCD_URL_EPEL     | EPEL repository URL      | https://dl.fedoraproject.org    |
| SPCD_URL_FEDORA   | Fedora repository URL    | https://rpmfind.net             |
| SPCD_URL_OPENSUSE | OpenSUSE repository URL  | https://download.opensuse.org   |
| SPCD_URL_PYTHON   | Python repository URL    | https://pypi.org                |
| SPCD_URL_ROCKY    | Rocky repository URL     | https://dl.rockylinux.org       |
| SPCD_URL_UBUNTU   | Ubuntu repository URL    | https://ubuntu.mirrors.ovh.net  |

### 3.3 [Operating Systems containers](#what) {#what-systems}

#### Latest

| os | https | up ca | python | ffmpeg | gource | graphviz | plantuml | shellcheck | shfmt | shunit |
|:----------------|---|---|------------:|-------:|-----:|------:|-----------:|------:|-----:|------:|
| Arch 20240818   | ☑ | ☑ |        3.12 |  7.0.2 | 0.54 | 12.0  |  1.2023.13 |  0.10 |  3.8 | 2.1.8 |
| Alpine 3.20     | ☑ | ☐ |        3.12 |  6.1.1 | 0.54 |  9.0  |  1.2024.4  |  0.10 |  3.8 | 2.1.8 |
| Fedora 40       | ☑ | ☑ | 3.12 → 3.13 |  6.1.2 | 0.55 |  9.0  |  1.2024.6  |  0.9  |  3.7 | 2.1.6 |
| Debian Bookworm | ☐ | ☐ |        3.11 |  5.1.6 | 0.54 |  2.42 |  1.2020.2  |  0.9  |  3.6 | 2.1.8 |
| OpenSUSE 15.6   | ☐ | ☑ | 3.6  → 3.12 |  4.4.4 | 0.54 |  2.48 |  1.2020.9  |  0.8  |  3.5 | 2.1.6 |
| Ubuntu Noble    | ☐ | ☐ |        3.12 |  6.1.1 | 0.54 | u2.42 | u1.2020.2  | u0.9  | u3.8 | 2.1.8 |
| Alma / Rocky 9  | ☑ | ☑ | 3.9  → 3.12 | e5.1.4 |      |  2.44 | e1.2024.6  | e0.8  |      |       |

#### Previous

| os | https | up ca | python | ffmpeg | gource | graphviz | plantuml | shellcheck | shfmt | shunit |
|:----------------|---|---|------------:|-------:|-----:|------:|-----------:|------:|-----:|------:|
| Alpine 3.19     | ☑ | ☐ |        3.11 |  6.1.1 | 0.54 |  9.0  |  1.2023.12 |  0.9  |  3.7 | 2.1.8 |
| Fedora 39       | ☑ | ☑ | 3.12 → 3.13 |  6.1.1 | 0.55 |  8.1  |  1.2024.6  |  0.9  |  3.5 | 2.1.6 |
| OpenSUSE 15.5   | ☐ | ☑ | 3.6  → 3.11 |  4.4.4 | 0.54 |  2.48 |  1.2020.9  |  0.8  |  3.5 | 2.1.6 |
| Alma / Rocky 8  | ☑ | ☑ | 3.6  → 3.12 |        |      |  2.40 | e1.2024.6  | e0.6  |      |       |

#### Older Python

| os | https | up ca | python | ffmpeg | gource | graphviz | plantuml | shellcheck | shfmt | shunit |
|:----------------|---|---|------------:|-------:|-----:|------:|-----------:|------:|-----:|------:|
| Ubuntu Jammy    | ☐ | ☐ |        3.10 |  4.4.2 | 0.51 | u2.42 | u1.2020.2  | u0.8  | u3.4 | 2.1.6 |
| Debian Bullseye | ☐ | ☐ |        3.9  |  4.3.7 | 0.51 |  2.42 |  1.2020.2  |  0.7  |      | 2.1.6 |

---

## 4 [Who](#toc) {#who}

* Author: [Marc Beninca](https://forge.rwx.work/marc.beninca)

---

## 5 [Where](#toc) {#where}

### 5.1 [Chat](#where) {#where-chat}

* [Discord](https://discord.com/channels/983145051985154108/1255894420092485652)
* [IRC](ircs://irc.libera.chat/#spcd)

### 5.2 [Repo](#where) {#where-repo}

* [Code](https://forge.rwx.work/rwx.work/spcd)
* [Feed](https://forge.rwx.work/rwx.work/spcd.rss)
* [Actions](https://forge.rwx.work/rwx.work/spcd/actions)

### 5.3 [Site](#where) {#where-site}

* [Web](https://spcd.rwx.work)

---

## 6 [When](#toc) {#when}

### 6.1 [Release tasks](#when) {#when-release}

#### Documentation

* build instead of readme
* generate graphs
* list behavior differences
  * Continuous Integration platforms
  * Operating Systems

#### Shell

* comment
* git/pypi fallback for rwx/spcd
* handle errors
* packages
  * configure
    * apk
    * pacman
    * zypper
  * repositories
    * codecs
      * epel
      * fedora
      * opensuse
    * opensuse
      * disable & enable https
* locales
  * persist
* test
* workspace variable

#### Shell → Python

* check file & variable sums
* install system packages
  * epel
    * plantuml
    * shellcheck
  * graphviz
  * openssh
  * rsync
  * shfmt
  * shunit
* python virtual environment
  * install system package
  * configure pip
  * venv
    * create
    * activate
* query package
* relay environment module name

#### Python

* check
  * .py
  * .sh
    * maximum line length
* comment
* detect ssh private key type
* fit banner to text
* implement substeps
* list sys.path
* set workflow templates
  * actions
  * includes
* try git repo url variable first for shunit
* test

### 6.2 [Further tasks](#when) {#when-further}

* automate versions fetching
* gource, xvfb, xauth
* handle openh264 repositories
* link from workspace to actions root
* rpm fusion
* tex
* translate to french
* try to support
  * guix
  * nix
* uv
